################################################################
#
#
#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation.
#
#     This program is distributed in the hope that it will be useful, but
#     WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program; if not, write to the Free Software
#     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#     The author's contact information:
#     
#----------------------------------------------------------------
#     Xia Wang
#     Department of Mathematical Sciences
#     University of Cincinnati, 
#     Cincinnati, OH 45221, USA
#     Office: 4428E French Hall West
#     Phone: 513-556-3295
#     Website: http://homepages.uc.edu/~wang2x7/
#     Email: xia.wang@uc.edu
#
#     Sep. 19, 2011
#----------------------------------------------------------------
#################################################################################

route<-"the\\work\\space\\"  # specify the workspace where the data is and where you'd like to save the plots and results.
#  read in data
#  All study 8 data are read in
# in the Algorithm Demo, only two runs are used
#####################################################################
study8ltq<-read.csv(file=paste(route,"data\\study8_ltq.abs.csv", sep=""),header=TRUE)
study8ltq$Peptide.ion<-as.character(study8ltq$Peptide.ion)
study8orbis<-read.csv(file=paste(route,"data\\study8_orbis.abs.csv", sep=""),header=TRUE)
study8orbis$Peptide.ion<-as.character(study8orbis$Peptide.ion)
study8<-merge(study8ltq, study8orbis, by=c("Peptide.ion"), all.x=TRUE, all.y=TRUE)
study8$RT<-ifelse(is.na(study8$RT..y), study8$RT..x, study8$RT..y)
study8$mz<-ifelse(is.na(study8$precursor.m.z..y), study8$precursor.m.z..x, study8$precursor.m.z..y)
study8$length<-ifelse(is.na(study8$length.y), study8$length.x, study8$length.y)
study8$numHKR<-ifelse(is.na(study8$num.HKR.y), study8$num.HKR.x, study8$num.HKR.y)
study8$mprotons<-ifelse(is.na(study8$mobile.protons.y), study8$mobile.protons.x, study8$mobile.protons.y)
study8$zlength<-(study8$mprotons+study8$numHKR-1)/sqrt(study8$length)

# subset study 8 data

study8.1<-subset(study8, select=c("Peptide.ion","RT","mz","length","mprotons","zlength",
                                  paste("abR",seq(1,18),".x", sep=""),
                                  paste("abR",seq(1,18),".y", sep=""), "abR19")
                                  )

colnames(study8.1)<-c("Peptide.ion","RT","mz","length","mprotons","zlength",
                                  paste("abR",seq(1,37),sep=""))
                                  
# define the corresponding concentration level and labs
conctr<-c("L","L","H",
          "L","H","H",
          "L","L","L",
          "H","H","H",
          "L","L","L",
          "H","H","H",
           "L","L","L",
          "H","H","H",
          "L","L","L",
          "H","H","H",
          "L","H","L","H",
          "L","L","H")
lab<-c(rep("ltq95",6),rep("ltq73",6), rep("ltq65",6),
       rep("orbi86",6),rep("orbi65",6), rep("orbi56",7))

l.con<-list()
l.con$ltq95<-c(1,2,4)
l.con$ltq73<-c(7,8,9)
l.con$ltq65<-c(13,14,15)
l.con$orbi86<-c(19, 20, 21)
l.con$orbi65<-c(25, 26, 27)
l.con$orbi56<-c(31,33, 35, 36)
h.con<-list()
h.con$ltq95<-c(3,5,6)
h.con$ltq73<-c(10,11,12)
h.con$ltq65<-c(16,17,18)
h.con$orbi86<-c(22,23,24)
h.con$orbi65<-c(28, 29, 30)
h.con$orbi56<-c(32,34,37)   
 # pick two runs for normalization
pick.contr<-c("high","low","highlow")
record.1<-NULL

#for (lab.run in 1:6){ 
 
#for high
#for (h.con.run.num in 1:(length(h.con[[lab.run]])-1)){
#run.num<-h.con[[lab.run]][h.con.run.num]
# for (h.con.run.den in (h.con.run.num+1):length(h.con[[lab.run]])){
#run.den<-h.con[[lab.run]][h.con.run.den] 


#for low
#for (l.con.run.num in 1:(length(l.con[[lab.run]])-1)){
#run.num<-l.con[[lab.run]][l.con.run.num]
# for (l.con.run.den in (l.con.run.num+1):length(l.con[[lab.run]])){
#run.den<-l.con[[lab.run]][l.con.run.den] 

# for differnt levels
#for (l.con.run in 1:length(l.con[[lab.run]])){
#run.num<-l.con[[lab.run]][l.con.run]
#for (h.con.run in 1:length(h.con[[lab.run]])){
#run.den<-h.con[[lab.run]][h.con.run]

run.num<-3
run.den<-8
lab.num<-lab[run.num]
lab.den<-lab[run.den]
contr.num<-conctr[run.num]
contr.den<-conctr[run.den]
pair<-subset(study8.1, select=c("Peptide.ion","RT","mz","length","zlength",
                              "mprotons",
                              paste("abR",run.num, sep=""),paste("abR",run.den, sep="")))
colnames(pair)<-c("peptide","RT","mz","length","zlength","mprotons","run1","run2")
pair.1<-subset(pair, !(is.na(run1)|is.na(run2)))
# define the abundance and ratio
pair.1$A<-0.5*(log10(pair.1$run1)+log10(pair.1$run2))
if (contr.num==contr.den) pair.1$M<-log2(pair.1$run1/pair.1$run2)
if (contr.num=="H"&contr.den=="L") pair.1$M<-log2(pair.1$run1/pair.1$run2)-log2(5)
if (contr.num=="L"&contr.den=="H") pair.1$M<-log2(pair.1$run1/pair.1$run2)+log2(5)
str(pair.1)
pair.2<-subset(pair.1, select=c("M", "RT","mz","A","length","zlength","mprotons"))

# read in the functions 
source(paste(route, "univariate p-spline.r", sep=""))

# correlation plot if preferred 
#pairs(pair.2, upper.panel = panel.smooth, lower.panel = panel.cor, 
#        diag.panel = panel.hist)


# fit the mixed model and simple linear model to M: select the single most important variable
# stepwisely select the most important variable

library(reshape)
 var.names<-c("M","RT","mz","A","length","zlength","mprotons")
 M.matrix<-matrix(NA, nrow(pair.2), 7)
 M.matrix[,1]<-pair.2$M
 RSS.1<-sum((pair.2$M)^2)
 RSS.all<-rep(0, 7)
 model.all<-rep(NA, 6) 
 RSS.all[1]<-RSS.1
 vars<-var.names[2:7]
 var.select<-rep(NA, 6)
 
 # use the A for normalization only
 
 RSS.A<-RSS(predictor=pair.2$A,response=M.matrix[,1],
                  pred.fit=pair.2$A,resp.fit=M.matrix[,1])
 M.A<-RSS.A$resid
                  
 # iteratively select the most important variable
 for ( n.step in 1:6){
 for (n.var in 1:length(vars)){
 print(n.var)
 loc<-which(var.names==vars[n.var])
 RSS.results<-RSS(predictor=pair.2[,loc],response=M.matrix[,n.step],
                  pred.fit=pair.2[,loc],resp.fit=M.matrix[,n.step]) 
pdf(paste(route,colnames(pair.2)[loc],"_", n.step,".pdf", sep=""),width =4, height = 3)
 par(mai=c(0.5, 0.5, 0.5, 0.1))
 plot(pair.2[,loc], M.matrix[,n.step],
#xlab=colnames(pair.2)[loc],ylab=paste("step ", n.step,": M",sep=""),
xlab="", ylab="",
  main=paste("D=", round(sum((RSS.results$resid)^2), digit=1)),cex.main=2) 
 univ.pspline(pair.2[,loc], M.matrix[,n.step])
dev.off()
 if (RSS.results$RSS>RSS.all[n.step+1]) {
 RSS.all[n.step+1]<-RSS.results$RSS
 model.all[n.step]<-RSS.results$model
 M.matrix[,n.step+1]<-RSS.results$resid
 var.select[n.step]<-vars[n.var]
 }
 }
 vars.1<-vars[-which(vars==var.select[n.step])]
 vars<-vars.1
 }
 # stepwise normalized value
 M.matrix
 
 # summary
 RSE<-sum(M.matrix[,7]^2)
 M.after<-M.matrix[,7]
 M.before<-M.matrix[,1]
 model.comp.out<-list()
 model.comp.out$var<-c("null",var.select,"RSE")
 model.comp.out$RSS<-c(RSS.all, RSE)
 model.comp.out$model<-c("NA",model.all, "NA")
 model.comp.out<-as.data.frame(model.comp.out)
 model.comp.out$pct<-model.comp.out$RSS/RSS.1
 model.comp.out$index<-c("NA",seq(1, 6),"NA")
 model.comp.out
 
 #plot(pair.2$RT, pair.2$M,xlab="RT",ylab="M", main=paste(SSE=) 
 #univ.pspline(pair.2$RT, pair.2$M)


#combine the normalization results together

#M.before.after.A<-cbind(pair.1$peptide, M.before, M.after, M.A)
#colnames(M.before.after.A)<-c("peptide","before","after","Aonly")
#write.csv(M.before.after.A, 
#file=
#paste(route,"results\\samelab\\highlow\\normalization\\",
#run.num,"_",run.den,".csv", sep=""))

#write.csv(model.comp.out, 
#file=
#paste(route,"results\\samelab\\highlow\\",
#run.num,"_",run.den,".csv", sep=""))

#write.csv(M.matrix, 
#file=
#paste(route,"results\\samelab\\highlow\\mmatrix\\",
#run.num,"_",run.den,".csv", sep=""))

#record.temp<-t(c(run.num, lab[run.num], run.den, lab[run.den],nrow(pair.2)))
#record.temp<-as.data.frame(record.temp)
#record.1<-rbind.fill(record.1,record.temp) 

#}}}

#record.1<-as.data.frame(record.1)
#colnames(record.1)<-c("run.num","lab.num", "run.den","lab.den","total.peptide")
#write.csv(record.1, file=paste(route,"\\results\\samelab\\highlow.count.csv", sep=""))    

