# this is a program for univariate mixed model
# for penalized spline regression


univ.pspline<-function(predictor, response){

# step 1 define the variables
x<-predictor
y<-response

# step 2 select the knots
default.knots <- function(x,num.knots)
{
if (missing(num.knots))
num.knots <- max(5,min(floor(length(unique(x))/4),35))
return(quantile(unique(x),seq(0,1,length=
(num.knots+2))[-c(1,(num.knots+2))]))
}
knots <- default.knots(x)

# step 3 create Z matrix
z <- outer(x, knots, "-") #Number of obs. X Number of knots
z <- z * (z > 0) #All negative elements of z set to 0

# step 4 prepare the model
group <- rep(1, length(x)) #Create the dummy grouping variable
#temp.dataframe <- data.frame(x, y)
#Combine the raw data into a data
#frame
#model.data <- grouped.data(y~x|group, data=temp.dataframe)
#Create the grouped data object
#for the fixed effects

# estimate the parameters
library(nlme)
#fit <- lme(y~x, random=pdIdent(~-1+z), data=model.data)

fit <- lme(y~x, random=list(group=pdIdent(~-1+z)) )
diag.AIC<-AIC(fit)
#X<-cbind(rep(1,length(x)), x)
#fit <- lme(y~-1+X, random=pdIdent(~-1+z|group) )

# plot the results

fit.beta.hat <- fit$coef$fixed
#extract the beta-hats
fit.u.hat <- unlist(fit$coef$random)
#extract the u-hats
#Construct the plot dimensions
x.grid <- seq(min(x), max(x), length=length(x))
X.grid <- cbind(1, x.grid)
Z.grid <- outer(x.grid, knots, "-")
Z.grid <- Z.grid * (Z.grid > 0)
yhat.grid <- X.grid%*%fit.beta.hat + Z.grid%*%fit.u.hat
#plot(x, y, type='p', xlab=xlab, ylab=ylab)
lines(x.grid, yhat.grid, col="red", lwd=3)
}
# some inference



# model AIC

diag.AIC<-function(predictor, response){

# step 1 define the variables
x<-predictor
y<-response
linear.model<-lm(y~x)
linear.AIC<-AIC(linear.model)
# step 2 select the knots
default.knots <- function(x,num.knots)
{
if (missing(num.knots))
num.knots <- max(5,min(floor(length(unique(x))/4),35))
return(quantile(unique(x),seq(0,1,length=
(num.knots+2))[-c(1,(num.knots+2))]))
}
knots <- default.knots(x)

# step 3 create Z matrix
z <- outer(x, knots, "-") #Number of obs. X Number of knots
z <- z * (z > 0) #All negative elements of z set to 0

# step 4 prepare the model
group <- rep(1, length(x)) #Create the dummy grouping variable
#temp.dataframe <- data.frame(x, y)
#Combine the raw data into a data
#frame
#model.data <- grouped.data(y~x|group, data=temp.dataframe)
#Create the grouped data object
#for the fixed effects

# estimate the parameters
library(nlme)
#fit <- lme(y~x, random=pdIdent(~-1+z), data=model.data)

fit <- lme(y~x, random=list(group=pdIdent(~-1+z)) )
diag.AIC<-AIC(fit)
model.comp<-ifelse(linear.AIC>diag.AIC, "pspline","linear")
model.comp.AIC<-ifelse(linear.AIC>diag.AIC, diag.AIC, linear.AIC)
model.comp.out<-list()
model.comp.out$AIC<-model.comp.AIC
model.comp.out$model<-model.comp
return(model.comp.out)}

# gam plots


univ.gam<-function(predictor, response){

# step 1 define the variables
x<-predictor
y<-response

fit<-gam(y~x)



# plot the results

fit.y <- fitted.values(fit)

lines(x, fit.y, col="red", lwd=3)
}
# some inference


# stepwise model selection the RSS calcualtion

RSS<-function(predictor, response, pred.fit, resp.fit){
RSS.results<-list()
# step 1 define the variables
x<-predictor
y<-response

xx<-pred.fit
yy<-resp.fit
# the linear model
linear.model<-lm(y~x)
linear.RSS<-sum((fitted.values(linear.model))^2)
linear.resid<-as.vector(y-fitted.values(linear.model))
# step 2 select the knots
default.knots <- function(x,num.knots)
{
if (missing(num.knots))
num.knots <- max(5,min(floor(length(unique(x))/4),35))
return(quantile(unique(x),seq(0,1,length=
(num.knots+2))[-c(1,(num.knots+2))]))
}
knots <- default.knots(x)

# step 3 create Z matrix
z <- outer(x, knots, "-") #Number of obs. X Number of knots
z <- z * (z > 0) #All negative elements of z set to 0

zz <- outer(xx, knots, "-") #Number of obs. X Number of knots
zz <- zz * (zz > 0) #All negative elements of z set to 0
# step 4 prepare the model
group <- rep(1, length(x)) #Create the dummy grouping variable
#temp.dataframe <- data.frame(x, y)
#Combine the raw data into a data
#frame
#model.data <- grouped.data(y~x|group, data=temp.dataframe)
#Create the grouped data object
#for the fixed effects

# estimate the parameters
library(nlme)
#fit <- lme(y~x, random=pdIdent(~-1+z), data=model.data)

fit <- lme(y~x, random=list(group=pdIdent(~-1+z)) )
diag.RSS<-sum((fitted.values(fit))^2)
diag.resid<-as.vector(y-fitted.values(fit))
model.comp<-ifelse(linear.RSS<diag.RSS, "pspline","linear")
model.comp.RSS<-ifelse(linear.RSS<diag.RSS, diag.RSS, linear.RSS)
if (model.comp=="pspline"){model.comp.resid<-diag.resid} else {
model.comp.resid<-linear.resid}
RSS.results$RSS<-model.comp.RSS
RSS.results$model<-model.comp
RSS.results$resid<-model.comp.resid
#Construct the plot dimensions
xx.grid <- xx
XX.grid <- cbind(1, xx.grid)
ZZ.grid <- outer(xx.grid, knots, "-")
ZZ.grid <- ZZ.grid * (ZZ.grid > 0)
if (model.comp=="pspline"){
# the spline fit to all data
fit.beta.hat <- fit$coef$fixed
#extract the beta-hats
fit.u.hat <- unlist(fit$coef$random)
#extract the u-hats
yyhat.grid <- XX.grid%*%fit.beta.hat + ZZ.grid%*%fit.u.hat
all.resid<-as.vector(yy-yyhat.grid)} else {
linear.beta.hat<-coef(linear.model)
yyhat.grid<-XX.grid%*%linear.beta.hat
all.resid<-as.vector(yy-yyhat.grid)}
RSS.results$resid.all<-all.resid
return(RSS.results)}


# functions for plots

panel.cor <- function(x, y, digits=2, prefix="", cex.cor, ...)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(0, 1, 0, 1))
    r <- cor(x, y)
    txt <- format(c(r, 0.123456789), digits=digits)[1]
    txt <- paste(prefix, txt, sep="")
    if(missing(cex.cor)) cex.cor <- 0.8/strwidth(txt)
    text(0.5, 0.5, txt, cex = max(cex.cor * abs(r)*1.5,1))
}
panel.hist <- function(x, ...)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(usr[1:2], 0, 1.5) )
    h <- hist(x, plot = FALSE)
    breaks <- h$breaks; nB <- length(breaks)
    y <- h$counts; y <- y/max(y)
    rect(breaks[-nB], 0, breaks[-1], y, col="cyan", ...)
}

panel.smooth<-function (x, y, col = par("col"), bg = NA, pch = par("pch"), 
    cex = 1, ...) 
{    points(x, y, pch = pch, col = col, bg = bg, cex = 0.2)
     univ.pspline(x,y)
#        lines(stats::lowess(x[ok], y[ok], f = span, iter = iter), 
#            col = col.smooth, ...)
}
